@@ -6,7 +6,7 @@
 #'                       \itemize{
 #'                         \item \strong{\code{alpha}}: Named list. Alpha parameters for each alternative, including for any outside good. As many elements as alternatives.
 #'                         \item \strong{\code{alternatives}}: Character vector. Names of alternatives, elements must match the names in list 'utilities'.
-#'                         \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1. These can be scalars or vectors (of length equal to rows in the database). A user can also specify \code{avail=1} to indicate universal availability, or omit the setting completely.
+#'                         \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1. These can be scalars or vectors (of length equal to rows in the database). A user can also specify \code{avail=1} to indicate universal availability.
 #'                         \item \strong{\code{budget}}: Numeric vector. Budget for each observation.
 #'                       \item \strong{\code{componentName}}: Character. Name given to model component. If not provided by the user, Apollo will set the name automatically according to the element in \code{P} to which the function output is directed.
 #'                         \item \strong{\code{continuousChoice}}: Named list of numeric vectors. Amount of consumption of each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
@@ -364,6 +364,10 @@ apollo_mdcev <- function(mdcev_settings,functionality){
     # Check that sigma is not random
     if(!is.vector(s$sigma)) stop("INCORRECT FUNCTION/SETTING USE - Forecasting not available for MDCEV with random sigma")
     
+    # Check that alpha-gamma profile is used if there is an outside good
+    isAG <- all( sapply(s$alpha[-1], \(a) all(a==s$alpha[[1]])) )
+    if(!isAG) stop("INCORRECT FUNCTION/SETTING USE - Forecasting not available for MDCEV with alpha-profile")
+    
     # Generate draws for Gumbel error components
     if(!is.null(apollo_inputs$apollo_control$seed)) seed <- apollo_inputs$apollo_control$seed + 5 else seed <- 13 + 5
     set.seed(seed)
